"use strict";(self["webpackChunkjs"]=self["webpackChunkjs"]||[]).push([[443],{2108:function(e,t,n){n.r(t),n.d(t,{default:function(){return c}});var s=function(){var e=this,t=e.$createElement;e._self._c;return e._m(0)},u=[function(){var e=this,t=e.$createElement,n=e._self._c||t;return n("div",{staticClass:"about"},[n("h1",[e._v("This is an about page")])])}],a=n(3736),l={},r=(0,a.Z)(l,s,u,!1,null,null,null),c=r.exports}}]);