/* General Purpose */

.clickable:hover {
    cursor: pointer;
}

.color-square { width: 17.4px; height: 17.4px; }

/* Fonts */
@font-face {
    font-family: LatoRegular;
    src: url(third-party/fonts/Lato-Regular.ttf);
  }
  
  @font-face {
    font-family: LatoLight;
    src: url(third-party/fonts/Lato-Light.ttf);
  }
  
  @font-face {
    font-family: LatoBold;
    src: url(third-party/fonts/Lato-Bold.ttf);
  }

/* Main */

body {
    overscroll-behavior: none;
    background-color: rgb(255, 255, 255);
    color: #4D4D4D;
}

h5 {
    font-family: LatoBold;
    font-size: 16px;
}

h6 {
    font-family: LatoRegular;
    font-size: 14px;
}

/* Main Navigation bar */
#controls {
    position: fixed;
    top: 106px;
    right: 0;
    box-sizing: border-box;
    background-color: #F3F8FB;
    width: 280px;
    height: calc(100% - 56px);
    padding-top: 24px;
    padding-left: 12px;
    padding-right: 12px;
    overflow-y: auto;
    overflow-x: hidden;
}

#navbar-top {
    position: fixed;
    top:45px;
    height: 60px;
    right: 0%;
    width: 100%;
    background-color: #ebebeb;
    box-shadow: 0 4px 8px 0 rgba(0, 0, 0, 0.2), 0 4px 5px 0 rgba(0, 0, 0, 0.19);
}

#projectAndFileSelection {
    width: 400px;
    display: inline-block;
    vertical-align: middle;
}

#metadata {
    /* position: fixed; 
    display: inline-block;
    vertical-align: middle; */
    font-family: LatoLight;
    font-size: 12px;
    /* left: 720px; 
    top: 55px;  */
}

#plot-graphs {
    display: inline-block;
    position: fixed; 
    left: 320px; 
    top: 61px; 
    color: #4D4D4D;
}

#zoomControl {
    display: inline-block;
    position: fixed; 
    left: 500px; 
    top: 61px;
    color: #4D4D4D;
}

#zoom-buttons {
    display: inline-block;
    position: fixed; 
    left: 600px; 
    top: 61px;
}

.btn-light {
    color: #4D4D4D;
    background-color: #ffffff;
    border-color: #ffffff;
    font-family: LatoRegular;
    font-size: 12px;
}

.bg-light {
    background-color: #F3F8FB!important;
    height: 45px;
}

button {
    background-color: #D9EAF4;
    border: 1px solid #c3e4f7;
    border-radius: 3px;
    font-family: LatoRegular;
    font-size: 12px;
    color: #4D4D4D;
}

.btn-light:hover{
    background-color: #D9EAF4;
}

.btn-primary {
    background-color: #C3E4F7;
    border-color: #C3E4F7;
    font-family: LatoRegular;
    font-size: 12px;
    color: #4D4D4D;
}

.btn-secondary {
    background-color: #4D4D4D;
    border-color: #4D4D4D;
    font-family: LatoRegular;
    font-size: 12px;
    color: #ffffff;
}

.btn-danger {
    background-color: #D95B5B;
    border-color: #D95B5B;
    font-family: LatoRegular;
    font-size: 12px;
    color: #ffffff;
}


.filter-option-inner-inner{
    font-family: LatoRegular;
    font-size: 12px;
    color: #4D4D4D;
}


#playbackControl { width: 100%; margin-bottom: 10px; }
#playbackControl > div { width: 100%; text-align: center; margin-bottom: 4px; }

.control-panel-table { font-size: 0.8em; }
.control-panel-table th { font-size: 0.9em; }

div.bootstrap-select.bs-width-correction > div.dropdown-menu { width: 100%; }
div.bootstrap-select > div.dropdown-menu ul.dropdown-menu { font-size: 0.875rem !important; }
div.bootstrap-select > div.dropdown-menu a.dropdown-item { padding: 4px 8px; }

#assignmentPanelArea { display: none; }
.assignment-panel {
    box-sizing: border-box;
    width: 100%;
    border: 1px solid #ccc;
    padding: 8px;
}
.assignment-panel > h6 { font-size: 0.9em; }
.assignment-panel .btn-group button { font-size: 0.7em; }
p.assignment-panel-bottom-label { font-size: 0.6em; margin-top: 0; font-style: italic; text-align: right; padding-right: 2px; }

#graphs {
    box-sizing: border-box;
    margin-bottom: 100px;
    position: fixed;
    top: 130px;
    left: 0px;
    width: calc(100% - 270px);
    height: calc(100% - 56px);
    padding: 8px 4px 60px 8px;
    overflow-y: auto;
    overflow-x: hidden;
}

#metadata {
    width: 100%;
    margin-bottom: 14px;
}

/* Matching dygraphs headers for now */
#metadata > h2 {
    font-size: 20px;
    text-align: center;
    font-weight: bold;
}

.graph_wrapper {
    width: 99%;
    height: 120px;
}

.graph_wrapper > table {
    width: 100%;
    height: 100%;
    table-layout: fixed;
}

.graph_wrapper > table > tbody > tr > td.graph_title, .graph_wrapper > table > tbody > tr > td.legend {
    width: 120px;
    padding-left: 4px;
}

/* DOUBLE CHECK */
.graph_wrapper > table > tbody > tr > td.graph_title {
    vertical-align: bottom;
    overflow: hidden;
    height: 60px;
    width: 55px;
    padding-bottom: 4px;
    padding-left: 15px;
    font-family: LatoBold;
    font-size: 12px;
    color: #4D4D4D;
    font-weight: bold;
    overflow-wrap: break-word;
}

.graph_wrapper > table > tbody > tr > td.graph_title > div {
    overflow: hidden;
    text-overflow: ellipsis;
    display: -webkit-box;
    -webkit-line-clamp: 2;
    -webkit-box-orient: vertical;
}

.graph_wrapper > table > tbody > tr > td.graph_title > .graph_subtitle {
    font-weight: normal;
    font-size: .8em;
    margin-top: 8px;
    word-wrap: break-word;
}

.graph_wrapper > table > tbody > tr > td.graph_title > .webix_icon {
    color: rgb(0, 98, 235);
    visibility: hidden;
    cursor: pointer;
    margin-left: 8px;
}

.graph_wrapper > table > tbody > tr:hover > td.graph_title > .webix_icon {
    visibility: visible !important;
}

.graph_wrapper > table > tbody > tr > td.legend {
    vertical-align: top;
    /*height: 120px;*/
    padding-top: 4px;;
}

.graph_wrapper > table > tbody > tr > td.legend > div {
    position: relative;
    display: inline-block;
    background-color: #F3F8FB;
    filter: drop-shadow(-2px 2px 2px #ccc);
    border: 1px solid #c3e4f7;
    padding: 2px 4px;
    word-wrap: break-word;
    border-radius: 5px;
    z-index: 99999;
    font-family: LatoRegular;
    font-size: 12px;
}

.graph_wrapper > table > tbody > tr > td.legend > div > span {
    white-space: nowrap;
}

.graph {
    width: 100%;
    /*height: 120px;*/
    height: 100%;
}

/* Realtime */

body.realtime > #controls #projectAndFileSelection, body.realtime > #controls > #patternDetectionPanel {
    display: none !important;
}
body.realtime #realtimeTitle {
    /*display: initial !important;*/
    display: inline !important;
}

#control_footer {
    width: 100%;
    clear: both;
    text-align: center;
    margin-top: 30px;
}

#control_footer > p#enterrealtime, #control_footer > p#exitrealtime {
    cursor: pointer;
    color: #888;
    font-size: 80%;
}

body:not(.realtime) > #controls > #control_footer > p#enterrealtime {
    display: block;
}

body:not(.realtime) > #controls > #control_footer > p#exitrealtime {
    display: none !important;
}

body.realtime > #controls > #control_footer > p#enterrealtime {
    display: none !important;
}

body.realtime > #controls > #control_footer > p#exitrealtime {
    display: block;
}

/* Annotations */

.form-group{
    font-family: LatoBold;
    font-size: 14px;
    color: #4D4D4D;
}

.annotationStartTime{
    font-family: LatoBold;
    font-size: 14px;
    color: #4D4D4D;
}

.form-control:disabled{
    background-color: #F3F8FB;
    font-family: LatoRegular;
    font-size: 13px;
    color: #4D4D4D;
}

.form-control{
    font-family: LatoRegular;
    font-size: 13px;
    color: #4D4D4D;
}

#allAnnotationsListModal tr:hover, #annotationsListModal tr:hover {
    cursor: pointer;
}

#annotationStartDate, #annotationStartTime, #annotationEndDate, #annotationEndTime {
    width: 85%;
    margin-bottom: 3px;
}

.webix_message_area {
    display: none !important;
}

#interactionsModal table {
    width: 100%;
}

#interactionsModal table > tbody > tr > td:first-child {
    padding-right: 24px;
}

/* AUViewer 2.0 Index page CSS */

.table {
    vertical-align: bottom;
    color: #4D4D4D;
    font-family: LatoRegular;
    border-collapse: collapse;

}

.table thead {
    background-color: #D9EAF4;
    text-align: center;
    font-family: LatoBold;
}

.table thead th{
    border:solid rgb(255, 255, 255) 1px;
    border-collapse: separate;
    padding: 6px;

}

td:first-child, th:first-child {
     border-left: none;
     border-radius:30px 0px 0px 30px;
}

td:last-child, th:last-child {
    border-left: none;
    border-radius: 0px 40px 40px 0px;
}

.row {
    padding-top: 150px;
    font-family: LatoBold;
    color: #4D4D4D;
}

.navbar-light .navbar-brand {
    color: #4D4D4D;
    font-family: LatoRegular;
    font-size: 15px;
}


.bootstrap-select>.dropdown-toggle {
    position: relative;
    width: 200px;
    text-align: right;
    white-space: nowrap;
}


.webix_view {
    -webkit-appearance: none;
    -webkit-font-smoothing: antialiased;
    background-color: #fff;
    border: 0 solid #dadee0;
    color: #475466;
    cursor: default;
    font-family: LatoRegular;
    font-size: 13px;
    letter-spacing: .2px;
    overflow: hidden;
    white-space: normal;
}

:not(.input-group)>.bootstrap-select.form-control:not([class*=col-]) {
    width: 200px;
    left: 10px;
}
/*  */

